@@ -2,8 +2,6 @@
 // for details. All rights reserved. Use of this source code is governed by a
 // BSD-style license that can be found in the LICENSE file.
 
-// @dart = 2.6
-
 // For the purposes of the mirrors library, we adopt a naming
 // convention with respect to getters and setters.  Specifically, for
 // some variable or field...
@@ -60,6 +58,7 @@
 library dart.mirrors;
 
 import 'dart:async' show Future;
+import "dart:_internal" show Since;
 
 /**
  * A [MirrorSystem] is the main interface used to reflect on a set of
@@ -110,6 +109,12 @@ abstract class MirrorSystem {
    */
   TypeMirror get voidType;
 
+  /**
+   * A mirror on the [:Never:] type.
+   */
+  @Since("2.8")
+  TypeMirror get neverType;
+
   /**
    * Returns the name of [symbol].
    *
@@ -132,7 +137,7 @@ abstract class MirrorSystem {
    * Using this method may result in larger output.  If possible, use
    * the const constructor of [Symbol] or symbol literals.
    */
-  external static Symbol getSymbol(String name, [LibraryMirror library]);
+  external static Symbol getSymbol(String name, [LibraryMirror? library]);
 }
 
 /**
@@ -151,7 +156,7 @@ external MirrorSystem currentMirrorSystem();
  * function can only be used to obtain  mirrors on objects of the current
  * isolate.
  */
-external InstanceMirror reflect(Object reflectee);
+external InstanceMirror reflect(dynamic reflectee);
 
 /**
  * Reflects a class declaration.
@@ -184,7 +189,7 @@ external ClassMirror reflectClass(Type key);
  * function can only be used to obtain type mirrors on types of the current
  * isolate.
  */
-external TypeMirror reflectType(Type key, [List<Type> typeArguments]);
+external TypeMirror reflectType(Type key, [List<Type>? typeArguments]);
 
 /**
  * A [Mirror] reflects some Dart language entity.
@@ -221,7 +226,7 @@ abstract class IsolateMirror implements Mirror {
    * 2. the isolate being reflected by this mirror is the same isolate being
    *    reflected by [other].
    */
-  bool operator ==(other);
+  bool operator ==(Object other);
 
   /**
    * Loads the library at the given uri into this isolate.
@@ -293,7 +298,7 @@ abstract class DeclarationMirror implements Mirror {
    * * For a parameter, local variable or local function the owner is the
    *   immediately enclosing function.
    */
-  DeclarationMirror get owner;
+  DeclarationMirror? get owner;
 
   /**
    * Whether this declaration is library private.
@@ -345,7 +350,7 @@ abstract class DeclarationMirror implements Mirror {
    *
    * This operation is optional and may throw an [UnsupportedError].
    */
-  SourceLocation get location;
+  SourceLocation? get location;
 
   /**
    * A list of the metadata associated with this declaration.
@@ -401,12 +406,8 @@ abstract class ObjectMirror implements Mirror {
    * If the invocation throws an exception *e* (that it does not catch), this
    * method throws *e*.
    */
-  /*
-   * TODO(turnidge): Handle ambiguous names.
-   * TODO(turnidge): Handle optional & named arguments.
-   */
-  InstanceMirror invoke(Symbol memberName, List positionalArguments,
-      [Map<Symbol, dynamic> namedArguments]);
+  InstanceMirror invoke(Symbol memberName, List<dynamic> positionalArguments,
+      [Map<Symbol, dynamic> namedArguments = const <Symbol, dynamic>{}]);
 
   /**
    * Invokes a getter and returns a mirror on the result.
@@ -469,8 +470,7 @@ abstract class ObjectMirror implements Mirror {
    * If the invocation throws an exception *e* (that it does not catch) this
    * method throws *e*.
    */
-  /* TODO(turnidge): Handle ambiguous names.*/
-  InstanceMirror setField(Symbol fieldName, Object value);
+  InstanceMirror setField(Symbol fieldName, dynamic value);
 
   /**
    * Performs [invocation] on the reflectee of this [ObjectMirror].
@@ -529,7 +529,7 @@ abstract class InstanceMirror implements ObjectMirror {
    * If you access [reflectee] when [hasReflectee] is false, an
    * exception is thrown.
    */
-  get reflectee;
+  dynamic get reflectee;
 
   /**
    * Whether this mirror is equal to [other].
@@ -545,7 +545,7 @@ abstract class InstanceMirror implements ObjectMirror {
    *    b. the remote objects reflected by this mirror and by [other] are
    *    identical.
    */
-  bool operator ==(other);
+  bool operator ==(Object other);
 }
 
 /**
@@ -597,8 +597,8 @@ abstract class ClosureMirror implements InstanceMirror {
    * If the invocation throws an exception *e* (that it does not catch), this
    * method throws *e*.
    */
-  InstanceMirror apply(List positionalArguments,
-      [Map<Symbol, dynamic> namedArguments]);
+  InstanceMirror apply(List<dynamic> positionalArguments,
+      [Map<Symbol, dynamic> namedArguments = const <Symbol, dynamic>{}]);
 }
 
 /**
@@ -630,7 +630,7 @@ abstract class LibraryMirror implements DeclarationMirror, ObjectMirror {
    * 2. The library being reflected by this mirror and the library being
    *    reflected by [other] are the same library in the same isolate.
    */
-  bool operator ==(other);
+  bool operator ==(Object other);
 
   /**
    * Returns a list of the imports and exports in this library;
@@ -656,17 +656,17 @@ abstract class LibraryDependencyMirror implements Mirror {
 
   /// Returns the library mirror of the library that is imported or exported,
   /// or null if the library is not loaded.
-  LibraryMirror get targetLibrary;
+  LibraryMirror? get targetLibrary;
 
   /// Returns the prefix if this is a prefixed import and `null` otherwise.
-  Symbol get prefix;
+  Symbol? get prefix;
 
   /// Returns the list of show/hide combinators on the import/export
   /// declaration.
   List<CombinatorMirror> get combinators;
 
   /// Returns the source location for this import/export declaration.
-  SourceLocation get location;
+  SourceLocation? get location;
 
   List<InstanceMirror> get metadata;
 
@@ -781,7 +781,7 @@ abstract class ClassMirror implements TypeMirror, ObjectMirror {
    *
    * If this type is [:Object:], the superclass will be null.
    */
-  ClassMirror get superclass;
+  ClassMirror? get superclass;
 
   /**
    * A list of mirrors on the superinterfaces of the reflectee.
@@ -845,11 +845,6 @@ abstract class ClassMirror implements TypeMirror, ObjectMirror {
    */
   ClassMirror get mixin;
 
-  // TODO(ahe): What about:
-  // /// Finds the instance member named [name] declared or inherited in the
-  // /// reflected class.
-  // DeclarationMirror instanceLookup(Symbol name);
-
   /**
    * Invokes the named constructor and returns a mirror on the result.
    *
@@ -878,8 +873,9 @@ abstract class ClassMirror implements TypeMirror, ObjectMirror {
    * * If evaluating the expression throws an exception *e* (that it does not
    *   catch), this method throws *e*.
    */
-  InstanceMirror newInstance(Symbol constructorName, List positionalArguments,
-      [Map<Symbol, dynamic> namedArguments]);
+  InstanceMirror newInstance(
+      Symbol constructorName, List<dynamic> positionalArguments,
+      [Map<Symbol, dynamic> namedArguments = const <Symbol, dynamic>{}]);
 
   /**
    * Whether this mirror is equal to [other].
@@ -892,7 +888,7 @@ abstract class ClassMirror implements TypeMirror, ObjectMirror {
    * Note that if the reflected class is an invocation of a generic class, 2.
    * implies that the reflected class and [other] have equal type arguments.
    */
-  bool operator ==(other);
+  bool operator ==(Object other);
 
   /**
    * Returns whether the class denoted by the receiver is a subclass of the
@@ -952,7 +948,7 @@ abstract class TypeVariableMirror extends TypeMirror {
    * 1. [other] is a mirror of the same kind, and
    * 2. [:simpleName == other.simpleName:] and [:owner == other.owner:].
    */
-  bool operator ==(other);
+  bool operator ==(Object other);
 }
 
 /**
@@ -983,7 +979,7 @@ abstract class MethodMirror implements DeclarationMirror {
   /**
    * The source code for the reflectee, if available. Otherwise null.
    */
-  String get source;
+  String? get source;
 
   /**
    * A list of mirrors on the parameters for the reflectee.
@@ -1085,7 +1081,7 @@ abstract class MethodMirror implements DeclarationMirror {
    * 1. [other] is a mirror of the same kind, and
    * 2. [:simpleName == other.simpleName:] and [:owner == other.owner:].
    */
-  bool operator ==(other);
+  bool operator ==(Object other);
 }
 
 /**
@@ -1131,7 +1127,7 @@ abstract class VariableMirror implements DeclarationMirror {
    * 1. [other] is a mirror of the same kind, and
    * 2. [:simpleName == other.simpleName:] and [:owner == other.owner:].
    */
-  bool operator ==(other);
+  bool operator ==(Object other);
 }
 
 /**
@@ -1171,7 +1167,7 @@ abstract class ParameterMirror implements VariableMirror {
    *
    * Returns `null` for a required parameter.
    */
-  InstanceMirror get defaultValue;
+  InstanceMirror? get defaultValue;
 }
 
 /**
